/*
  mkmkman.c  --  make mkman.c
  Copyright (C) 1999 Steffen Solyga <solyga@absinth.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include	"header.h"
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>
#define		MKMKMAN_BUFFER_SIZE	32

/*
  - reads manual template from stdin, writes mkman.c to stdout
  - if a substring "[FORMAT DEF_NAME]" is found it is replaced by (an
    equivalent printf() command for) the value of the constant DEF_NAME,
    where FORMAT will be used as output format for printf()
    example: "[%s name_of_this_program]"
  - DEF_NAME must not contain a ']'
  ==> "[%" indicates the substring to replace

last modified: Sat Mar 20 23:24:48 CET 1999 @beast

*/


char*
read_line( char* s ) {
  static size_t size= MKMKMAN_BUFFER_SIZE;
  size_t nbr= 0;
  int c;
  if( s == NULL ) s= (char*)malloc(size);
  if( s == NULL ) return( NULL );
  while( (c=getchar()) != EOF ) {
    s[nbr++]= (char)c;
    if( c == '\n' ) {
      s[nbr-1]= '\0';	/* string end */
      s[nbr]=   '\0';	/* buffer end */
      break;
    }
    if( nbr+1 >= size ) {	/* +1 for buffer end */
      size+= MKMKMAN_BUFFER_SIZE;
      s= realloc( s, size );
      if( s == NULL ) return( NULL );
    }
  }
  if( nbr == 0 ) return( NULL );
  return( s );
}


int
output_normal_printf( char* s ) {
  char* p= s;
  if( *s == '\0' ) return( 0 );
  printf( "  printf( \"" );
  while( *p != '\0' ) {
    switch( *p ) {
      case '"' : printf( "\\%c", *p ); break;
      case '\\': printf( "\\%c", *p ); break;
      case '%' : printf( "%%%c", *p ); break;
      default  : printf( "%c", *p );
    }
    p++;
  }
  printf( "\" );\n" );
  return( 0 );
}


int
main( int argc, char** argv ) {
  int retval= RETVAL_OK;
  char* line_str= NULL;
  int line= 0;
  char actual_prg_name[]= prg_name;
  char* p= actual_prg_name;

/* output header stuff */
  printf( "/*\n" );
  printf( "  mkman.c  --  output manual page for %s\n", prg_name );
  printf( "  generated automatically by %s\n", *argv );
  printf( "  Copyright (C) 1999 Steffen Solyga <solyga@absinth.net>\n" );
  printf( "*/\n" );
  printf( "\n" );
  printf( "#include\t\"header.h\"\n" );
  printf( "#include\t<stdio.h>\n" );
  printf( "\n" );
  /* program name(s) */
  *p= (char)toupper((int)*p);
  printf( "#define\tPrg_name\t\"%s\"\n", actual_prg_name );
  while( *p != '\0' ) { *p= (char)toupper((int)*p); p++; }
  printf( "#define\tPRG_NAME\t\"%s\"\n", actual_prg_name );
  /* channels */
  printf( "#define\tHELP_CHANNEL_STR\t\"%s\"\n",
          HELP_CHANNEL==stdout?"stdout":"stderr" );
  printf( "#define\tVERSION_CHANNEL_STR\t\"%s\"\n", 
          VERSION_CHANNEL==stdout?"stdout":"stderr" );
  printf( "#define\tVERBOSE_CHANNEL_STR\t\"%s\"\n", 
          VERBOSE_CHANNEL==stdout?"stdout":"stderr" );
  printf( "#define\tERROR_CHANNEL_STR\t\"%s\"\n",
          ERROR_CHANNEL==stdout?"stdout":"stderr" );
  /* manual section */
  printf( "#define\tMANUAL_SECTION_STR\t\"%s ", OS_TYPE_STR );
  switch( MANUAL_NUMBER ) {
    case  1: printf( "User's " ); break;
    case  2:
    case  3: printf( "Programmer's " ); break;
    case  8: printf( "System Manager's " ); break;
    default: ;
  }
  printf( "Manual\"\n" );
  printf( "\n" );
  /* main() header */
  printf( "int main( int argc, char* argv ) {\n" );
  printf( "  int retval= RETVAL_OK;\n" );

/* output body stuff (man page contents) */
  while( (line_str=read_line(line_str)) != NULL ) {
    char* s= line_str;
    char* p;
    line++;
    while( (p=strstr(s,"[%")) != NULL ) {
      char* pend;
      char* fmt;
      if( (pend=strstr(p,"]")) == NULL ) {
        fprintf( stderr, "%s: Missing `]' (input line %d)\n",
                 *argv, line );
        retval= RETVAL_ERROR; goto DIE_NOW;
      }
      *p= '\0'; p++;			/* delete '[', *p == '%' */
      *pend= '\0';			/* delete ']' */
      output_normal_printf( s );
      s= pend+1;			/* s -> behind ']' (now '\0') */
      if( (pend=strstr(p," ")) == NULL ) {
        fprintf( stderr, "%s: Missing space in `%s' (input line %d)\n",
                 *argv, p, line );
        retval= RETVAL_ERROR; goto DIE_NOW;
      }
      *pend= '\0';			/* delete ' ' */
      fmt= p;
      p= pend+1;			/* p -> DEF_NAME */
      printf( "  printf( \"%s\", %s );\n", fmt, p );
    }
    output_normal_printf( s );
    printf( "  printf( \"\\n\" );\n" );
  }

/* output tail stuff */
DIE_NOW:
  printf( "  exit( retval );\n" );
  printf( "}\n" );
  free( line_str );
  exit( retval );
}
